//@+leo
//@+node:0::@file src/config-gui.c
//@+body
//@@language c
/*
    rox-tasklist    provides a tasklist for the rox desktop
    Copyright (C) 2002  Robert Lowe

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    The author may be contacted at pngwen@acm.org
*/

//@<< config-gui #includes >>
//@+node:1::<< config-gui #includes >>
//@+body
#include "config-gui.h"
#include "globals.h"
#include "task-gui.h"
#include <choices.h>
#include <stdio.h>
#include <string.h>

//@-body
//@-node:1::<< config-gui #includes >>


//@<< config-gui declarations >>
//@+node:2::<< config-gui declarations >>
//@+body


/* ignore data type */
#define IS 0
#define ISNOT 1
#define DOESCONTAIN 2
#define DOESNOTCONTAIN 3
#define BEGINSWITH 4
#define ENDSWITH 5
struct config_ignore_type
{
  char* search_string;
  int search_type;
};
typedef struct config_ignore_type config_ignore_type;

/* control types for the ignore windows */
GtkWidget* ignore_list;
GtkWidget* ignore_wname_entry;
GtkWidget* ignore_mode_menu;
GtkWidget* ignore_mode_selector;
GtkWidget* ignore_update_button;
GtkWidget* ignore_delete_button;



/* global configuration data */
GtkWidget* confwin = NULL;
config_options_type temp_config_options;
GtkWidget* colorwin = NULL;
GtkWidget* colorbut = NULL;
GtkWidget* preview;


/* prototypes */
void configure_ok(GtkWidget* widget, gpointer data);
void configure_cancel(GtkWidget* widget, gpointer data);
void configure_apply(GtkWidget* widget, gpointer data);
GtkWidget* configure_build_options();
GtkWidget* configure_build_text_color();
GtkWidget* configure_build_button_color();
void configure_color(GtkWidget* widget, gpointer color);
void configure_color_ok(GtkWidget* widget, gpointer color);
void configure_color_cancel(GtkWidget* widget, gpointer color);
GtkWidget* configure_build_ignore_windows();
GtkWidget* configure_build_display_mode();
void configure_build_display_mode_scrollbar_option(GtkWidget* widget, gpointer data);
void configure_build_display_mode_nonscrollbar_option(GtkWidget* widget, gpointer data);
GtkWidget* configure_build_display_width();
GtkWidget* configure_build_display_width_preview();
GtkWidget* configure_build_bug_options();
void configure_build_bug_options_toggle(GtkWidget* widget, gpointer data);
void configure_build_display_width_changed(GtkWidget* widget, gpointer data);
void configure_save_choices();
void configure_apply_options();
void configure_button_color_change(GtkWidget* button, GdkColor color);
void configure_load();
void ignore_windows_add(GtkWidget* widget, gpointer data);
void ignore_windows_select_row(GtkCList* clist, 
                               gint row,
							      gint column,
								  GdkEventButton event,
								  gpointer data);
void ignore_windows_unselect_row(GtkCList* clist, 
                               gint row,
							      gint column,
								  GdkEventButton event,
								  gpointer data);
char* ignore_windows_mode_text(int mode);
void ignore_windows_delete(GtkWidget* widget, gpointer data);
void ignore_windows_update(GtkWidget* widget, gpointer data);
GList* ignore_list_copy(GList* source);
void ignore_list_delete(GList* list);
void config_save_ignore_list();
void config_load_ignore_list();

//@-body
//@-node:2::<< config-gui declarations >>


//@+others
//@+node:3::Creation and Initialization
//@+node:1::configure
//@+body
//@+at
//  creates the configure window

//@-at
//@@c

void configure(GtkWidget* window, gpointer data)
{
  GtkWidget* button;
  GtkWidget* winbox;
  GtkWidget* box;
  GtkTooltips* tooltips;

  /* first get rid of temp_config's ignore list */
  ignore_list_delete(temp_config_options.ignore_windows);
  
  /* do the stuff via shallow copy */
  temp_config_options = config_options;
  
  /* copy over the ignore_list */
  temp_config_options.ignore_windows=
                           ignore_list_copy(config_options.ignore_windows);
  

  /* prevent multiple confwins from coming into play */
  if (confwin)
  {
    gtk_widget_show(confwin);
    return;
  }

  tooltips = gtk_tooltips_new();

  confwin = gtk_window_new(GTK_WINDOW_DIALOG);
  gtk_window_set_policy(GTK_WINDOW(confwin), FALSE, FALSE, TRUE);
  gtk_signal_connect_object(GTK_OBJECT(confwin), "destroy",
                            GTK_SIGNAL_FUNC(configure_cancel), NULL);

  winbox = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(confwin), winbox);

  /* This was based on the code for showing the main window but has been replaced by
     the code below. The code in the main window should be replaced by the code below
     to hopefully remove the crash on scroll bug.
  view = gtk_viewport_new(0, 0);
  gtk_widget_set_usize(view, 250, 300);
  vscroller = gtk_vscrollbar_new(gtk_viewport_get_vadjustment(GTK_VIEWPORT(view)));
  viewbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start_defaults(GTK_BOX(viewbox), view);
  gtk_box_pack_start_defaults(GTK_BOX(viewbox), vscroller);
  gtk_box_pack_start_defaults(GTK_BOX(winbox), viewbox);
  gtk_container_add(GTK_CONTAINER(view), configure_build_options());
  */
  /* ditch the scrolled window if we're going for tabs in a notebook, but keep this as I still need
     to a reminder to trade out some code as described in the comment above
  scrolledwin = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_set_usize(scrolledwin, 300, 300);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start(GTK_BOX(winbox), scrolledwin, TRUE, TRUE, 0);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolledwin), configure_build_options());
  */
  gtk_box_pack_start(GTK_BOX(winbox), configure_build_options(), FALSE, FALSE, 5);

  gtk_box_pack_start(GTK_BOX(winbox), gtk_label_new("Choices will be saved as $HOME/Choices/rox-tasklist/..."), FALSE, FALSE, 5);

  box = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(winbox), box, TRUE, TRUE, 5);

  button = gtk_button_new_with_label("Apply");
  gtk_widget_set_usize(button, 60, 25);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(configure_apply), NULL);
  gtk_tooltips_set_tip(tooltips, button, "Apply new settings", NULL);
  gtk_box_pack_end(GTK_BOX(box), button, FALSE, FALSE, 5);

  button = gtk_button_new_with_label("Cancel");
  gtk_widget_set_usize(button, 60, 25);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(configure_cancel), NULL);
  gtk_tooltips_set_tip(tooltips, button, "Do not save and apply new settings", NULL);
  gtk_box_pack_end(GTK_BOX(box), button, FALSE, FALSE, 5);

  button = gtk_button_new_with_label("OK");
  gtk_widget_set_usize(button, 60, 25);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(configure_ok), NULL);
  gtk_tooltips_set_tip(tooltips, button, "Save and apply new settings", NULL);
  gtk_box_pack_end(GTK_BOX(box), button, FALSE, FALSE, 5);

  gtk_widget_grab_focus(button);

  gtk_object_set_data(GTK_OBJECT(confwin), "tooltips", tooltips);


  gtk_widget_show_all(confwin);
}

//@-body
//@+node:1::configure_ok
//@+body
//@+at
//  callback for configure window's ok button

//@-at
//@@c

void configure_ok(GtkWidget* widget, gpointer data)
{
  configure_apply_options();
  configure_save_choices();

  if (confwin)
    gtk_widget_destroy(confwin);
  confwin = NULL;
}

//@-body
//@-node:1::configure_ok
//@+node:2::configure_cancel
//@+body
//@+at
//  callback for configure window's cancel button

//@-at
//@@c

void configure_cancel(GtkWidget* widget, gpointer data)
{
  if (confwin)
    gtk_widget_destroy(confwin);
  confwin = NULL;

  /* debug goodies */
  /*
  g_print("mode=%d\n", config_options.mode);
  g_print("width=%d\n", config_options.width);
  g_print("\n");
  */
}

//@-body
//@-node:2::configure_cancel
//@+node:3::configure_apply
//@+body
//@+at
//  callback for configure window's apply button

//@-at
//@@c

void configure_apply(GtkWidget* widget, gpointer data)
{
  configure_apply_options();
  configure_save_choices();
}

//@-body
//@-node:3::configure_apply
//@-node:1::configure
//@+node:2::configure_defaults
//@+body
//@+at
//  set everything to "factory" defaults

//@-at
//@@c

void configure_defaults()
{
  GtkStyle* normal_style;


  /* initialize colors */
  normal_style=gtk_widget_get_default_style();

  /* text colors */
  config_options.text_color_normal=normal_style->text[GTK_STATE_NORMAL];
  config_options.text_color_focused=normal_style->text[GTK_STATE_NORMAL];
  /* make iconified text color */
  config_options.text_color_iconified.red=0xffff;
  config_options.text_color_iconified.green=0;
  config_options.text_color_iconified.blue=0;
  gdk_color_alloc(gdk_colormap_get_system(), 
		  &config_options.text_color_iconified);

  /* button colors */
  config_options.button_color_normal=normal_style->bg[GTK_STATE_NORMAL];
  config_options.button_color_iconified=normal_style->bg[GTK_STATE_NORMAL];
  config_options.button_color_focused=normal_style->bg[GTK_STATE_NORMAL];
  config_options.button_color_focused.red+=10000;
  config_options.button_color_focused.green+=10000;
  config_options.button_color_focused.blue+=10000;

  /* set the rest of the stuff */
  config_options.ignore_windows = NULL;  /* ignore nothing */
  config_options.mode = NONSCROLLED;
  config_options.width = DEFAULT_WIDTH;
}

//@-body
//@-node:2::configure_defaults
//@+node:3::configure_init
//@+body
//@+at
//  init config options from Choices

//@-at
//@@c

void configure_init()
{
  char* path;
  FILE* file;

  /* initialize the choices system */
  choices_init();

  path=choices_find_path_save("config", "rox-tasklist", TRUE); 
  file=fopen(path, "r");
  free(path);

  if(!file)
   configure_defaults();
  else 
   configure_load(file);
   
   
  /* initially there is no temp list */
  temp_config_options.ignore_windows=NULL;
}

//@-body
//@-node:3::configure_init
//@+node:4::configure_build_options
//@+body
//@+at
//  returns a box with configure options built

//@-at
//@@c

GtkWidget* configure_build_options()
{
  GtkWidget* mainbox;
  GtkWidget* notebook;
  GtkWidget* box;

  mainbox = gtk_hbox_new(FALSE, 5);
  notebook = gtk_notebook_new();
  gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook), TRUE);
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_LEFT);
  gtk_box_pack_start(GTK_BOX(mainbox), notebook, TRUE, TRUE, 5);

  box = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(box), configure_build_display_mode(), FALSE, FALSE, 5);
  gtk_box_pack_start(GTK_BOX(box), configure_build_display_width(), FALSE, FALSE, 5);
  gtk_box_pack_start(GTK_BOX(box), configure_build_display_width_preview(),
                     FALSE, FALSE, 5);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), box, gtk_label_new("Display options"));

  box = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start_defaults(GTK_BOX(box), configure_build_text_color());
  gtk_box_pack_start_defaults(GTK_BOX(box), configure_build_button_color());
  /*gtk_box_pack_start_defaults(GTK_BOX(box), configure_build_background_color());*/
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), box, gtk_label_new("Color options"));

  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), configure_build_ignore_windows(), gtk_label_new("Ignore windows"));

  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), configure_build_bug_options(), gtk_label_new("Bug options"));

  return mainbox;
}

//@-body
//@-node:4::configure_build_options
//@-node:3::Creation and Initialization
//@+node:4::Choices File Interface
//@+node:1::configure_load
//@+body
//@+at
//  load configuration from file

//@-at
//@@c

void configure_load(FILE* file)
{
  fscanf(file, "%hd\n%hd\n%hd\n", &config_options.text_color_normal.red,
	 &config_options.text_color_normal.green,
	 &config_options.text_color_normal.blue);

  fscanf(file, "%hd\n%hd\n%hd\n", &config_options.text_color_iconified.red,
	 &config_options.text_color_iconified.green,
	 &config_options.text_color_iconified.blue);

  fscanf(file, "%hd\n%hd\n%hd\n", &config_options.text_color_focused.red,
	 &config_options.text_color_focused.green,
	 &config_options.text_color_focused.blue);

  fscanf(file, "%hd\n%hd\n%hd\n", &config_options.button_color_normal.red,
	 &config_options.button_color_normal.green,
	 &config_options.button_color_normal.blue);

  fscanf(file, "%hd\n%hd\n%hd\n", &config_options.button_color_iconified.red,
	 &config_options.button_color_iconified.green,
	 &config_options.button_color_iconified.blue);

  fscanf(file, "%hd\n%hd\n%hd\n", &config_options.button_color_focused.red,
	 &config_options.button_color_focused.green,
	 &config_options.button_color_focused.blue);

  fscanf(file, "%d", &config_options.width);
  fscanf(file, "%d", &config_options.mode);

  fclose(file);
  
  /* load up the ignore list */
  config_load_ignore_list();
}

//@-body
//@-node:1::configure_load
//@+node:2::configure_save_choices
//@+body
//@+at
//  save options to Choices

//@-at
//@@c

void configure_save_choices()
{
  char* path;
  FILE* file;

  path=choices_find_path_save("config", "rox-tasklist", TRUE);

  file=fopen(path, "w");
  if(!file)
  {
    free(path);
    return;
  }

  fprintf(file, "%hd\n%hd\n%hd\n", config_options.text_color_normal.red,
	  config_options.text_color_normal.green,
	  config_options.text_color_normal.blue);

  fprintf(file, "%hd\n%hd\n%hd\n", config_options.text_color_iconified.red,
	  config_options.text_color_iconified.green,
	  config_options.text_color_iconified.blue);

  fprintf(file, "%hd\n%hd\n%hd\n", config_options.text_color_focused.red,
	  config_options.text_color_focused.green,
	  config_options.text_color_focused.blue);

  fprintf(file, "%hd\n%hd\n%hd\n", config_options.button_color_normal.red,
	  config_options.button_color_normal.green,
	  config_options.button_color_normal.blue);

  fprintf(file, "%hd\n%hd\n%hd\n", config_options.button_color_iconified.red,
	  config_options.button_color_iconified.green,
	  config_options.button_color_iconified.blue);

  fprintf(file, "%hd\n%hd\n%hd\n", config_options.button_color_focused.red,
	  config_options.button_color_focused.green,
	  config_options.button_color_focused.blue);

  fprintf(file, "%d\n", config_options.width);
  fprintf(file, "%d\n", config_options.mode);

  fclose(file);

  free(path);
  
  /* now, save the ignore list */
  config_save_ignore_list();
}

//@-body
//@-node:2::configure_save_choices
//@+node:3::config_save_ignore_list
//@+body
//@+at
//  save's the ignore list
//   The file format is stored in double lines as such
// 
//   search_string\n
//   search_type\n

//@-at
//@@c

void config_save_ignore_list()
{
  char* path;
  FILE* file;
  GList* cur;
  config_ignore_type* node;
  
  path=choices_find_path_save("ignore_list", "rox-tasklist", TRUE);
  file=fopen(path, "w");
  free(path);
  
  if(!file)
    return;

  cur=config_options.ignore_windows;
  while(cur)
  {
    node=(config_ignore_type*) cur->data;
    fprintf(file, "%s\n%d\n", node->search_string, node->search_type);
	 cur=cur->next;
  }
  fclose(file);
}
//@-body
//@-node:3::config_save_ignore_list
//@+node:4::config_load_ignore_list
//@+body
//@+at
//  reads in the ignore_window file
//   The file format is explained in the config_save_ignore_list node

//@-at
//@@c

void config_load_ignore_list()
{
  GString* search_string;
  config_ignore_type* node;
  char* path;
  FILE* file;
  char c;
  
  /* open up the choices file */
  path=choices_find_path_save("ignore_list", "rox-tasklist", TRUE);
  file=fopen(path, "r");
  free(path);
  
  if(!file)
    return;
	
  /* start the list off as empty */
  config_options.ignore_windows=NULL;
  
  while(!feof(file))
  {
    node=(config_ignore_type*) malloc(sizeof(config_ignore_type));
	
	 /* read in the window name */
	 c=fgetc(file);
	 search_string=g_string_sized_new(10);
	 while(c!='\n')
	 {
	   search_string=g_string_append_c(search_string, c);
	   c=fgetc(file);
	 }
	 node->search_string=search_string->str;
	 g_string_free(search_string, FALSE);
	 
	 /* read in the mode */
	 fscanf(file, "%d\n", &node->search_type);
	 
	 /* append it to the list */
	config_options.ignore_windows=g_list_append(config_options.ignore_windows,
	  (gpointer) node);
  }
  fclose(file);
}
//@-body
//@-node:4::config_load_ignore_list
//@-node:4::Choices File Interface
//@+node:5::Display Options
//@+node:1::configure_build_display_mode
//@+body
//@+at
//  returns a frame containing display mode configuration

//@-at
//@@c

GtkWidget* configure_build_display_mode()
{
  GtkWidget* frame;
  GtkWidget* box;
  GtkWidget* button_scrolled;
  GtkWidget* button_nonscrolled;

  frame = gtk_frame_new("Display mode");

  box = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(frame), box);

  button_scrolled = gtk_radio_button_new_with_label(NULL, "Scrollbar with free sized buttons");
  gtk_signal_connect(GTK_OBJECT(button_scrolled), "clicked",
                     GTK_SIGNAL_FUNC(configure_build_display_mode_scrollbar_option), NULL);
  gtk_box_pack_start_defaults(GTK_BOX(box), button_scrolled);

  button_nonscrolled = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(button_scrolled)), 
                                           "No scrollbar and buttons shrink to fit");
  gtk_signal_connect(GTK_OBJECT(button_nonscrolled), "clicked",
                     GTK_SIGNAL_FUNC(configure_build_display_mode_nonscrollbar_option), NULL);
  gtk_box_pack_start_defaults(GTK_BOX(box), button_nonscrolled);

  if (config_options.mode == SCROLLED)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button_scrolled), TRUE);
  else /* if (config_options.mode == NONSCROLLED) */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button_nonscrolled), TRUE);

  return frame;
}

//@-body
//@+node:1::configure_build_display_mode_scrollbar_option
//@+body
//@+at
//  set scrolled mode

//@-at
//@@c

void configure_build_display_mode_scrollbar_option(GtkWidget* widget, gpointer data)
{
  temp_config_options.mode = SCROLLED;
}

//@-body
//@-node:1::configure_build_display_mode_scrollbar_option
//@+node:2::configure_build_display_mode_nonscrollbar_option
//@+body
//@+at
//  set non-scrolled mode

//@-at
//@@c

void configure_build_display_mode_nonscrollbar_option(GtkWidget* widget, gpointer data)
{
  temp_config_options.mode = NONSCROLLED;
}

//@-body
//@-node:2::configure_build_display_mode_nonscrollbar_option
//@-node:1::configure_build_display_mode
//@+node:2::configure_build_display_width
//@+body
//@+at
//  returns a frame containing display width configuration

//@-at
//@@c

GtkWidget* configure_build_display_width()
{
  GtkWidget* frame;
  GtkWidget* box;
  GtkWidget* button;
  GtkWidget* label;
  GtkObject* adjustment;
  GtkTooltips* tooltips;

  tooltips = gtk_tooltips_new();

  frame = gtk_frame_new("Display width");

  box = gtk_hbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(frame), box);

  /* TODO: clean up the min/max settings. max should be something like the width of the desktop */
  adjustment = gtk_adjustment_new(config_options.width, 50, 10000, 1, 10, 0);
  button = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 0.0, 0);
  gtk_signal_connect(GTK_OBJECT(button), "changed", 
                     GTK_SIGNAL_FUNC(configure_build_display_width_changed), NULL);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(button), TRUE);
  gtk_tooltips_set_tip(tooltips, button, "Selects the width of the tasklist.", NULL);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 2);
  /*gtk_box_pack_start_defaults(GTK_BOX(box), button);*/

  label = gtk_label_new("pixels");
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 2);

  gtk_object_set_data(GTK_OBJECT(frame), "tooltips", tooltips);

  return frame;
}

//@-body
//@+node:1::configure_build_display_width_changed
//@+body
//@+at
//  set width on changed signal

//@-at
//@@c

void configure_build_display_width_changed(GtkWidget* widget, gpointer data)
{
  temp_config_options.width=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
  gtk_widget_set_usize(preview, temp_config_options.width, HEIGHT);
}

//@-body
//@-node:1::configure_build_display_width_changed
//@-node:2::configure_build_display_width
//@+node:3::configure_build_display_width_preview
//@+body
GtkWidget* configure_build_display_width_preview()
{
  GtkWidget* result;
  GtkStyle* style;
  GtkWidget* vp; 
  GtkWidget* frame;
  GtkWidget* scroll;
  GtkWidget* box;
  GtkWidget* note;
  int i;

  /* first build the scroller, view port and all that stuff */
  vp=gtk_viewport_new(NULL, NULL);
  scroll=gtk_hscrollbar_new(gtk_viewport_get_hadjustment(GTK_VIEWPORT(vp)));
  result=gtk_vbox_new(0,0);
  gtk_box_pack_start(GTK_BOX(result), vp, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(result), scroll, TRUE, TRUE,0);
  /*gtk_widget_set_usize(result, 500, HEIGHT+20);*/

  frame=gtk_frame_new("preview of window size");

  style=gtk_style_copy(gtk_widget_get_style(mainwin));

  for(i=0; i<5; i++)
  {
    style->bg[i]=config_options.text_color_normal;
    style->fg[i]=config_options.text_color_normal;
  }

  gtk_widget_set_style(frame, style);
  gtk_style_unref(style);

  gtk_widget_set_usize(frame, config_options.width, HEIGHT);

  box=gtk_hbox_new(FALSE, FALSE);

  gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);

  gtk_container_add(GTK_CONTAINER(vp), box);

  preview=frame;

  note=gtk_label_new("Note: When running as an applet in ROX 1.2.x, for some reason if you make the\nwindow smaller you must restart the applet by choosing quit and then dragging\nand dropping the applet to the panel to see the effect.");

  gtk_label_set_justify(GTK_LABEL(note), GTK_JUSTIFY_LEFT);
  gtk_box_pack_start(GTK_BOX(result), note, TRUE, TRUE,0);
 
  return result;
}

//@-body
//@-node:3::configure_build_display_width_preview
//@-node:5::Display Options
//@+node:6::Color Options
//@+node:1::configure_build_text_color
//@+body
//@+at
//  returns a frame containing text color configuration

//@-at
//@@c

GtkWidget* configure_build_text_color()
{
  GtkWidget* frame;
  GtkWidget* mainbox;
  GtkWidget* box;
  GtkWidget* button;
  GtkWidget* label;

  frame = gtk_frame_new("Text color");

  mainbox = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(frame), mainbox);

  box = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(mainbox), box, FALSE, FALSE, 2);
  button = gtk_button_new();
  gtk_widget_set_usize(button, 35, 20);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(configure_color), &temp_config_options.text_color_normal);
  configure_button_color_change(button, temp_config_options.text_color_normal);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 2);
  label = gtk_label_new("Normal button text color");
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 2);

  box = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(mainbox), box, FALSE, FALSE, 2);
  button = gtk_button_new();
  gtk_widget_set_usize(button, 35, 20);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(configure_color), &temp_config_options.text_color_iconified);
  configure_button_color_change(button, temp_config_options.text_color_iconified);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 2);
  label = gtk_label_new("Iconified button text color");
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 2);

  box = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(mainbox), box, FALSE, FALSE, 2);
  button = gtk_button_new();
  gtk_widget_set_usize(button, 35, 20);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(configure_color), &temp_config_options.text_color_focused);
  configure_button_color_change(button, temp_config_options.text_color_focused);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 2);
  label = gtk_label_new("Focused button text color");
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 2);

  return frame;
}

//@-body
//@-node:1::configure_build_text_color
//@+node:2::configure_build_button_color
//@+body
//@+at
//  returns a frame containing button color configuration

//@-at
//@@c

GtkWidget* configure_build_button_color()
{
  GtkWidget* frame;
  GtkWidget* mainbox;
  GtkWidget* box;
  GtkWidget* button;
  GtkWidget* label;

  frame = gtk_frame_new("Button color");

  mainbox = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(frame), mainbox);

  box = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(mainbox), box, FALSE, FALSE, 2);
  button = gtk_button_new();
  gtk_widget_set_usize(button, 35, 20);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(configure_color), &temp_config_options.button_color_normal);
  configure_button_color_change(button, temp_config_options.button_color_normal);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 2);
  label = gtk_label_new("Normal button color");
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 2);

  box = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(mainbox), box, FALSE, FALSE, 2);
  button = gtk_button_new();
  gtk_widget_set_usize(button, 35, 20);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(configure_color), &temp_config_options.button_color_iconified);
  configure_button_color_change(button, temp_config_options.button_color_iconified);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 2);
  label = gtk_label_new("Iconified button color");
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 2);

  box = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(mainbox), box, FALSE, FALSE, 2);
  button = gtk_button_new();
  gtk_widget_set_usize(button, 35, 20);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(configure_color), &temp_config_options.button_color_focused);
  configure_button_color_change(button, temp_config_options.button_color_focused);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 2);
  label = gtk_label_new("Focused button color");
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 2);

  return frame;
}

//@-body
//@-node:2::configure_build_button_color
//@+node:3::configure_button_color_change
//@+body
//@+at
//  change the color of a button

//@-at
//@@c

void configure_button_color_change(GtkWidget* button, GdkColor color)
{
  int i;
  GtkStyle* style;
  
  /* set the button color */
  style=gtk_style_copy(gtk_widget_get_style(mainwin));
  for(i=0; i<5; i++)
  {
    style->fg[i]=color;
    style->bg[i]=color;
    style->base[i]=color;
    style->text[i]=color;
  }

  gtk_widget_set_style(button, style);
  gtk_style_unref(style);
}

//@-body
//@-node:3::configure_button_color_change
//@+node:4::Color Dialog
//@+node:1::configure_color
//@+body
//@+at
//  sets a color

//@-at
//@@c

void configure_color(GtkWidget* widget, gpointer color)
{
  gdouble color_array[4];
  colorbut = widget;
  if(colorwin)
  {
    gtk_widget_destroy(colorwin);
  }
  colorwin = gtk_color_selection_dialog_new("Select color");

  /* populate the array */
  color_array[0]=(gdouble)((GdkColor*)color)->red / (gdouble) 0xffff;
  color_array[1]=(gdouble)((GdkColor*)color)->green / (gdouble) 0xffff;
  color_array[2]=(gdouble)((GdkColor*)color)->blue / (gdouble) 0xffff;
  color_array[3]=1;
  gtk_color_selection_set_color(GTK_COLOR_SELECTION(
				  GTK_COLOR_SELECTION_DIALOG(colorwin)->colorsel), 
				color_array);
  gtk_widget_show_all(colorwin);

  gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(colorwin)->ok_button),
                     "clicked", GTK_SIGNAL_FUNC(configure_color_ok), color);

  gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(colorwin)->cancel_button),
                     "clicked", GTK_SIGNAL_FUNC(configure_color_cancel), NULL);
}

//@-body
//@+node:1::configure_color_ok
//@+body
//@+at
//  handles ok button for color selection

//@-at
//@@c

void configure_color_ok(GtkWidget* widget, gpointer color)
{
  gdouble color_array[4];

  /* get the color */
  gtk_color_selection_get_color(GTK_COLOR_SELECTION(
				  GTK_COLOR_SELECTION_DIALOG(colorwin)->colorsel), 
				color_array);

  ((GdkColor*)color)->red=color_array[0] * 0xffff;
  ((GdkColor*)color)->green=color_array[1] * 0xffff;
  ((GdkColor*)color)->blue=color_array[2]  * 0xffff;

  /* loose the unneeded dialog */
  gtk_widget_destroy(colorwin);
  colorwin=0;

  /* allocate the color */
  gdk_color_alloc(gdk_colormap_get_system(), (GdkColor*) color);

  /* change the button color */
  configure_button_color_change(colorbut, *((GdkColor*) color));
}

//@-body
//@-node:1::configure_color_ok
//@+node:2::configure_color_cancel
//@+body
//@+at
//  handles cancel button for color selection

//@-at
//@@c

void configure_color_cancel(GtkWidget* widget, gpointer data)
{
  gtk_widget_destroy(colorwin);
  colorwin=0;
}

//@-body
//@-node:2::configure_color_cancel
//@-node:1::configure_color
//@-node:4::Color Dialog
//@-node:6::Color Options
//@+node:7::Ignore Windows
//@+node:1::configure_build_ignore_windows
//@+body
//@+at
//  returns a frame containing ignore windows configuration

//@-at
//@@c

GtkWidget* configure_build_ignore_windows()
{
  GtkWidget* vbox;
  GtkWidget* vbox2;
  GtkWidget* hbox;
  GtkWidget* add_button;
  GtkWidget* menuitem;
  GtkObject* adjustment;
  GtkWidget* ignore_scroller;
  config_ignore_type* node;
  GList* cur;
  char* text[2];
  int row;
  

  
  gchar* titles[]={"Mode", "Window Title"};
  

//@+at
//  Build the list widget

//@-at
//@@c  
  ignore_list=gtk_clist_new_with_titles(2, titles);
  gtk_widget_set_usize(ignore_list, 300,60);
  gtk_clist_set_column_auto_resize(GTK_CLIST(ignore_list), 0, TRUE);


//@+at
//  create the main containers

//@-at
//@@c
  vbox = gtk_vbox_new(FALSE, 5);
  hbox = gtk_hbox_new(FALSE, 0);
  

//@+at
//  Create the mode selection Menu and title

//@-at
//@@c
  ignore_mode_menu=gtk_menu_new();
  
  menuitem=gtk_menu_item_new_with_label("Is");
  gtk_object_set_data(GTK_OBJECT(menuitem), "ignore_window_mode", (gpointer)IS);
  gtk_menu_append(GTK_MENU(ignore_mode_menu), menuitem);

  menuitem=gtk_menu_item_new_with_label("Is Not");
  gtk_object_set_data(GTK_OBJECT(menuitem), "ignore_window_mode", (gpointer)ISNOT);
  gtk_menu_append(GTK_MENU(ignore_mode_menu), menuitem);

  menuitem=gtk_menu_item_new_with_label("Does Contain");
  gtk_object_set_data(GTK_OBJECT(menuitem), "ignore_window_mode", 
		      (gpointer)DOESCONTAIN);
  gtk_menu_append(GTK_MENU(ignore_mode_menu), menuitem);

  menuitem=gtk_menu_item_new_with_label("Does Not Contain");
  gtk_object_set_data(GTK_OBJECT(menuitem), "ignore_window_mode", 
		      (gpointer)DOESNOTCONTAIN);
  gtk_menu_append(GTK_MENU(ignore_mode_menu), menuitem);

  menuitem=gtk_menu_item_new_with_label("Begins With");
  gtk_object_set_data(GTK_OBJECT(menuitem), "ignore_window_mode", 
		      (gpointer)BEGINSWITH);
  gtk_menu_append(GTK_MENU(ignore_mode_menu), menuitem);

  menuitem=gtk_menu_item_new_with_label("Ends With");
  gtk_object_set_data(GTK_OBJECT(menuitem), "ignore_window_mode", 
		      (gpointer)ENDSWITH);
  gtk_menu_append(GTK_MENU(ignore_mode_menu), menuitem);



  ignore_mode_selector=gtk_option_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(ignore_mode_selector),
                           ignore_mode_menu);
  
  vbox2= gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start_defaults(GTK_BOX(vbox2), gtk_label_new("Mode"));
  gtk_box_pack_start_defaults(GTK_BOX(vbox2), ignore_mode_selector);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), vbox2);
  

//@+at
//  Create the Window Title entry

//@-at
//@@c
  vbox2= gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start_defaults(GTK_BOX(vbox2), gtk_label_new("Window Title"));
  ignore_wname_entry=gtk_entry_new();
  gtk_box_pack_start_defaults(GTK_BOX(vbox2), ignore_wname_entry);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), vbox2);  
  

//@+at
//  Pack the control section into the main vbox

//@-at
//@@c
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  


//@+at
//  Build the buttons for this thing

//@-at
//@@c
  hbox=gtk_hbox_new(FALSE, 0);
  add_button=gtk_button_new_with_label(" Add ");
  gtk_box_pack_start(GTK_BOX(hbox), add_button, FALSE, FALSE, 5);
  
  ignore_update_button=gtk_button_new_with_label(" Update ");
  gtk_box_pack_start(GTK_BOX(hbox), ignore_update_button, FALSE, FALSE, 0);
  gtk_widget_set_sensitive(ignore_update_button, FALSE);
  
  ignore_delete_button=gtk_button_new_with_label(" Delete ");
  gtk_box_pack_start(GTK_BOX(hbox), ignore_delete_button, FALSE, FALSE, 0);
  gtk_widget_set_sensitive(ignore_delete_button, FALSE);
  
  

//@+at
//  pack the buttons

//@-at
//@@c
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  

//@+at
//  Build the adjustments for the CList

//@-at
//@@c
  adjustment=gtk_adjustment_new(0,0,0,1,0,0);
  ignore_scroller=gtk_vscrollbar_new(GTK_ADJUSTMENT(adjustment));
  gtk_clist_set_vadjustment(GTK_CLIST(ignore_list),
                            GTK_ADJUSTMENT(adjustment));
  

  

//@+at
//  Pack the list widget and scrollbar into box

//@-at
//@@c
  hbox=gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), ignore_list);
  gtk_box_pack_start(GTK_BOX(hbox), ignore_scroller, FALSE, FALSE, 0);
  gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
  

//@+at
//  Here we set up all the call backs to the stuff we just made

//@-at
//@@c
  gtk_signal_connect(GTK_OBJECT(add_button), "clicked",
                     GTK_SIGNAL_FUNC(ignore_windows_add), NULL);
					 
  gtk_signal_connect(GTK_OBJECT(ignore_update_button), "clicked",
                     GTK_SIGNAL_FUNC(ignore_windows_update), NULL);
					 
  gtk_signal_connect(GTK_OBJECT(ignore_delete_button), "clicked",
                     GTK_SIGNAL_FUNC(ignore_windows_delete), NULL);

  gtk_signal_connect(GTK_OBJECT(ignore_list), "select-row",
                     GTK_SIGNAL_FUNC(ignore_windows_select_row), 
					   NULL);

  gtk_signal_connect(GTK_OBJECT(ignore_list), "unselect-row",
                     GTK_SIGNAL_FUNC(ignore_windows_unselect_row),
					   NULL);
					   

//@+at
//  now we add to our list widget the items in the temp_config list

//@-at
//@@c
  cur=temp_config_options.ignore_windows;
  while(cur)
  {
    node=(config_ignore_type*) cur->data;
    text[0]=ignore_windows_mode_text(node->search_type);
    text[1]=(char*) malloc(strlen(node->search_string)+1);
    strcpy(text[1], node->search_string);
    row=gtk_clist_append(GTK_CLIST(ignore_list), text);
  
    /* set the data for this row.  This is the mode for the row */
    gtk_clist_set_row_data(GTK_CLIST(ignore_list), row, 
	                        (gpointer) node->search_type);
    cur=cur->next;
  }

  return vbox;
}
//@-body
//@+node:1::ignore_windows_select_row
//@+body
//@+at
//  Signal handler for selecting rows

//@-at
//@@c

void ignore_windows_select_row(GtkCList* clist, 
                               gint row,
							      gint column,
								  GdkEventButton event,
								  gpointer data)
{
  gchar* text;
  int mode;
  
  gtk_widget_set_sensitive(ignore_update_button, TRUE);
  gtk_widget_set_sensitive(ignore_delete_button, TRUE);
  
  /* set text of the selection */
  gtk_clist_get_text(clist, row, 1, &text);
  gtk_entry_set_text(GTK_ENTRY(ignore_wname_entry), text);
  
  /* set the mode in the menu*/
  mode=(int) gtk_clist_get_row_data(clist, row);
  gtk_option_menu_set_history(GTK_OPTION_MENU(ignore_mode_selector), mode);
  
  /* add the row selected to the data keys in the clist */
  gtk_object_set_data(GTK_OBJECT(clist), "ignore_row", (gpointer) row);
}
//@-body
//@-node:1::ignore_windows_select_row
//@+node:2::ignore_windows_unselect_row
//@+body
//@+at
//  signal handler for unselecting a row

//@-at
//@@c

void ignore_windows_unselect_row(GtkCList* clist, 
                               gint row,
							      gint column,
								  GdkEventButton event,
								  gpointer data)
{
  gtk_widget_set_sensitive(ignore_update_button, FALSE);
  gtk_widget_set_sensitive(ignore_delete_button, FALSE);
  
}
//@-body
//@-node:2::ignore_windows_unselect_row
//@+node:3::ignore_windows_add
//@+body
//@+at
//  adds a row to the ignore windows list

//@-at
//@@c


void ignore_windows_add(GtkWidget* widget, gpointer data)
{
  char* text[2];
  char* tmp;
  int mode;
  int row;
  config_ignore_type* node;
  
  /* first set up the window name one */
  tmp=gtk_entry_get_text(GTK_ENTRY(ignore_wname_entry));
  text[1]=malloc(strlen(tmp)+1);
  strcpy(text[1], tmp);
  gtk_entry_set_text(GTK_ENTRY(ignore_wname_entry), "");
  
  
  /* now, make the mode field */ mode=(int)gtk_object_get_data(GTK_OBJECT(gtk_menu_get_active(GTK_MENU(ignore_mode_menu))), "ignore_window_mode");
  
  text[0]=ignore_windows_mode_text(mode);
    
  /* add to the list */
  row=gtk_clist_append(GTK_CLIST(ignore_list), text);
  
  /* set the data for this row.  This is the mode for the row */
  gtk_clist_set_row_data(GTK_CLIST(ignore_list), row, (gpointer) mode);
  gtk_option_menu_set_history(GTK_OPTION_MENU(ignore_mode_selector), 0);
  
  /* put it in the actual list */
  node=(config_ignore_type*) malloc(sizeof(config_ignore_type));
  node->search_type=mode;
  node->search_string=(char*) malloc(strlen(text[1])+1);
  strcpy(node->search_string, text[1]);
  temp_config_options.ignore_windows=
                         g_list_append(temp_config_options.ignore_windows,
						                 (gpointer) node);
}
//@-body
//@-node:3::ignore_windows_add
//@+node:4::ignore_windows_update
//@+body
//@+at
//  Updates the currently selected row

//@-at
//@@c

void ignore_windows_update(GtkWidget* widget, gpointer data)
{
  char* text[2];
  char* tmp;
  int mode;
  int row;
  config_ignore_type* node;
  GList* cur;
  
  /* first set up the window name one */
  tmp=gtk_entry_get_text(GTK_ENTRY(ignore_wname_entry));
  text[1]=malloc(strlen(tmp)+1);
  strcpy(text[1], tmp);
  gtk_entry_set_text(GTK_ENTRY(ignore_wname_entry), "");
  
  
  /* now, make the mode field */ mode=(int)gtk_object_get_data(GTK_OBJECT(gtk_menu_get_active(GTK_MENU(ignore_mode_menu))), "ignore_window_mode");
  
  text[0]=ignore_windows_mode_text(mode);
    
  /* update the list */
  row=(int)gtk_object_get_data(GTK_OBJECT(ignore_list), "ignore_row");
  gtk_clist_set_text(GTK_CLIST(ignore_list), row, 0, text[0]);
  gtk_clist_set_text(GTK_CLIST(ignore_list), row, 1, text[1]);
  
  /* set the data for this row.  This is the mode for the row */
  gtk_clist_set_row_data(GTK_CLIST(ignore_list), row, (gpointer) mode);
  gtk_option_menu_set_history(GTK_OPTION_MENU(ignore_mode_selector), 0);
  
  /* make the changes in the actual list */
  cur=g_list_nth(temp_config_options.ignore_windows, row);
  node=(config_ignore_type*)cur->data;
  free(node->search_string);
  node->search_type=mode;
  node->search_string=(char*) malloc(strlen(text[1])+1);
  strcpy(node->search_string, text[1]);
}
//@-body
//@-node:4::ignore_windows_update
//@+node:5::ignore_windows_delete
//@+body
//@+at
//  Deletes an item from the list

//@-at
//@@c

void ignore_windows_delete(GtkWidget* widget, gpointer data)
{
  int row;
  config_ignore_type* node;
  GList* cur;
  
  row=(int) gtk_object_get_data(GTK_OBJECT(ignore_list), "ignore_row");
  
  gtk_clist_remove(GTK_CLIST(ignore_list), row);
  
  gtk_entry_set_text(GTK_ENTRY(ignore_wname_entry), "");
  gtk_option_menu_set_history(GTK_OPTION_MENU(ignore_mode_selector), 0);
  
  cur=g_list_nth(temp_config_options.ignore_windows, row);
  temp_config_options.ignore_windows=g_list_remove_link(temp_config_options.ignore_windows, cur);
  
  node=(config_ignore_type*) cur->data;
  cur->data=0;
  
  free(node->search_string);
  free(node);
  
  g_list_free(cur);
}
//@-body
//@-node:5::ignore_windows_delete
//@-node:1::configure_build_ignore_windows
//@+node:2::ignore_list_copy
//@+body
//@+at
//  Does a deep copy of the source list, returning the pointer to the new list

//@-at
//@@c

GList* ignore_list_copy(GList* source)
{
  GList* list=NULL;
  GList* cur;
  config_ignore_type* node;
  config_ignore_type* cur_node;
  
  cur=source;
  while(cur)
  {
    /* get the node's data */
    cur_node=(config_ignore_type*) cur->data;
	/* allocate space for the node */
    node=(config_ignore_type*) malloc(sizeof(config_ignore_type));
	/* copy the search string */
	node->search_string=(char*) malloc(strlen(cur_node->search_string)+1);
	strcpy(node->search_string, cur_node->search_string);
	/* copy search type */
	node->search_type=cur_node->search_type;
	/* add it to the new list */
	list=g_list_append(list, (gpointer) node);
	/* go to the next node */
	cur=cur->next;
  }

  return list;
}
//@-body
//@-node:2::ignore_list_copy
//@+node:3::ignore_list_delete
//@+body
//@+at
//  Deletes the ignore_list, freeing all memory in the structs

//@-at
//@@c

void ignore_list_delete(GList* list)
{
  GList* cur;
  
  config_ignore_type* node;
  
  /* free the strings in the list */
  cur=list;
  while(cur)
  {
    node=(config_ignore_type*) cur->data;
    free(node->search_string);
    free(node);
    cur->data=0;
    cur=cur->next;
  }
  g_list_free(list);
}
//@-body
//@-node:3::ignore_list_delete
//@+node:4::ignore_list_check
//@+body
//@+at
//  Returns 1 if this window name should be ignored according to the rules
//   in the ignore list.  Returns 0 if the item should not be ignored
// 
//   The Search modes are as follows:

//@-at
//@@c
/*
 +---------------+---------------------------------------------------------+
 |Mode           |Explanation                                              |
 +---------------+---------------------------------------------------------+
 |IS             |wname is an EXACT match to the search_string of some node|
 |ISNOT          |wname is not a match to the search_string of some node   |
 |DOESCONTAIN    |wname does contain search_string of some node at some    |
 |               |  location in wname                                      |
 |DOESNOTCONTAIN |wname does not contain search_string of some node at any |
 |               |  location in wname                                      |
 |BEGINSWITH     |wname begins with the search_string of some node         |
 |ENDSWITH       |wname ends with the searc string of some node            |
 +---------------+---------------------------------------------------------+
*/

//@+at
// 
//  A window is ignored if just ONE node in the config structure's list matches
//  the wname

//@-at
//@@c

int ignore_list_check(char* wname)
{
  config_ignore_type* node;
  GList* cur;
  int len, len2;

  /* get length of the name in question */
  len=strlen(wname);

  /* start at the beginning of the list */
  cur=config_options.ignore_windows;

  /* check all nodes, returning one if we find a winner */
  while(cur)
  {
    node=(config_ignore_type*) cur->data;
    switch(node->search_type)
    {
      case IS:  if(!strcmp(wname, node->search_string)) return 1;
                break;

      case ISNOT:  if(strcmp(wname, node->search_string)) return 1;
                   break;

      case DOESCONTAIN:  if(strstr(wname, node->search_string)) return 1;
                         break;

      case DOESNOTCONTAIN: if(!strstr(wname, node->search_string)) return 1;
                           break;

      case BEGINSWITH:  len2=strlen(node->search_string);
                        if(len>=len2)
                        {
                          if(!strncmp(wname, node->search_string, len2))
                            return 1;
                        }
                        break;

      case ENDSWITH: len2=strlen(node->search_string);
                     if(len>=len2)
                     {
                       if(!strcmp(wname+(len-len2), node->search_string))
                         return 1;
                     }
                     break;

      default: break;
    }
    cur=cur->next;
  }

  /* didn't meet any conditions, so return */
  return 0;
}
//@-body
//@-node:4::ignore_list_check
//@+node:5::ignore_windows_mode_text
//@+body
//@+at
//  returns the string for the mode

//@-at
//@@c

char* ignore_windows_mode_text(int mode)
{

//@+at
//  These are the modes listed in the order that allows the mode number
//   to server as an index.

//@-at
//@@c

  char * mode_texts[]={"Is", "Is Not", "Does Contain", "Does Not Contain",
                       "Begins With", "Ends With"};
  char* result;
  
  result=malloc(strlen(mode_texts[mode]));
  strcpy(result, mode_texts[mode]);
  
  return result;
}
//@-body
//@-node:5::ignore_windows_mode_text
//@-node:7::Ignore Windows
//@+node:8::Bug Options
//@+node:1::configure_build_bug_options
//@+body
GtkWidget* configure_build_bug_options()
{
  GtkWidget* box;
  GtkWidget* button;

  box = gtk_vbox_new(FALSE, 5);

  button = gtk_check_button_new_with_label("Crash randomly with no explanation");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
  gtk_signal_connect(GTK_OBJECT(button), "toggled", 
                     GTK_SIGNAL_FUNC(configure_build_bug_options_toggle), NULL);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 2);

  button = gtk_check_button_new_with_label("Crash when scrolling fast on buttons");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
  gtk_signal_connect(GTK_OBJECT(button), "toggled", 
                     GTK_SIGNAL_FUNC(configure_build_bug_options_toggle), NULL);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 2);

  button = gtk_check_button_new_with_label("Sometimes, do not work properly with WM's other than sawfish");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
  gtk_signal_connect(GTK_OBJECT(button), "toggled", 
                     GTK_SIGNAL_FUNC(configure_build_bug_options_toggle), NULL);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 2);

  button = gtk_check_button_new_with_label("Do not work with rox-filer 1.3.0+");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
  gtk_signal_connect(GTK_OBJECT(button), "toggled", 
                     GTK_SIGNAL_FUNC(configure_build_bug_options_toggle), NULL);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 2);

  button = gtk_check_button_new_with_label("Seg fault when using GTK+ 2.0");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
  gtk_signal_connect(GTK_OBJECT(button), "toggled", 
                     GTK_SIGNAL_FUNC(configure_build_bug_options_toggle), NULL);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 2);

  return box;
}

//@-body
//@+node:1::configure_build_bug_options_toggle
//@+body
void configure_build_bug_options_toggle(GtkWidget* widget, gpointer data)
{
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
}

//@-body
//@-node:1::configure_build_bug_options_toggle
//@-node:1::configure_build_bug_options
//@-node:8::Bug Options
//@+node:9::configure_apply_options
//@+body
//@+at
//  applies options to the main window

//@-at
//@@c

void configure_apply_options()
{
  GList* cur;
  GList* cur2;

  /*empty the tasklist*/
  cur=tasklist;
  while(cur)
  {
    cur2=cur->next;
    remove_task((task_type*)cur->data);
    cur=cur2;
  }
  tasklist=0;

  /* handle width changes*/
  if(temp_config_options.width!=config_options.width)
  {
    if(!isapplet)
      gtk_widget_hide_all(mainwin);

    gtk_widget_set_usize(mainwin,
			 temp_config_options.width,
			 HEIGHT);
    if(!isapplet)
      gtk_widget_show_all(mainwin);
  }

  /* handle mode switches */
  if(temp_config_options.mode!=config_options.mode)
  {
    if(!isapplet)
      gtk_widget_hide_all(mainwin);

    /* make the changes */
    if(temp_config_options.mode==SCROLLED)
    {
      viewscroll=gtk_hscrollbar_new(gtk_viewport_get_hadjustment(GTK_VIEWPORT(view)));
      gtk_box_pack_start(GTK_BOX(viewbox), viewscroll, 0,0,0);
    }
    else
    {
      gtk_widget_hide(viewscroll);
      gtk_container_remove(GTK_CONTAINER(viewbox), viewscroll); 
      viewscroll=0;
    }
    if(!isapplet)
      gtk_widget_show_all(mainwin);

  }

  /* destroy the config_options ignore_windows */
  ignore_list_delete(config_options.ignore_windows);
  
  /* shallow copy configuration over */
  config_options=temp_config_options;
  
  /* deep copy the list */
  config_options.ignore_windows=
                        ignore_list_copy(temp_config_options.ignore_windows);



  /* rebuild the tasklist */
  make_task_list(1); 
}

//@-body
//@-node:9::configure_apply_options
//@-others


//@-body
//@-node:0::@file src/config-gui.c
//@-leo
