//@+leo
//@+node:0::@file src/task-gui.c
//@+body
//@@language c
/*
    rox-tasklist    provides a tasklist for the rox desktop
    Copyright (C) 2002  Robert Lowe

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    The author may be contacted at pngwen@acm.org
*/

//@<< task-gui #includes >>
//@+node:1::<< task-gui #includes >>
//@+body
#include <stdlib.h>
#include <stdio.h>
#include "task-gui.h"
#include <gdk-pixbuf/gdk-pixbuf.h>
#include"unknown.xpm"

//@-body
//@-node:1::<< task-gui #includes >>


//@<< task-gui declarations >>
//@+node:2::<< task-gui declarations >>
//@+body

GdkPixmap* unknown_pixmap=0;
GdkBitmap* unknown_bitmap=0;
GtkWidget* taskMenu;
taskbutton_type* current_task;

//@-body
//@-node:2::<< task-gui declarations >>


//@+others
//@+node:3::Task Window State Handlers
//@+body
//@+at
// 
// 
// These functions set various attributes of the button passed to them to
// reflect some window state of the task.  The states for which attributes
// exist are:
// 
// focused,
// unfocused,
// normal,
// iconified

//@-at
//@-body
//@+node:1::taskbutton_focus
//@+body
//@+at
//  set focus on this task button

//@-at
//@@c

void taskbutton_focus(taskbutton_type* task)
{
  int i;
  GtkStyle* style;

  /* do color stuff */
  style=gtk_style_copy(gtk_widget_get_style(mainwin));
  for(i=0; i<5; i++)
  {
    style->text[i]=config_options.text_color_focused;
    style->fg[i]=config_options.text_color_focused; 
    style->bg[i]=config_options.button_color_focused;
    style->base[i]=config_options.button_color_focused;
  }
  gtk_widget_set_style(task->label, style);
  gtk_widget_set_style(task->button, style);
  gtk_style_unref(style);

  /* show it */
  gtk_widget_show(task->button);
}

//@-body
//@-node:1::taskbutton_focus
//@+node:2::taskbutton_unfocus
//@+body
//@+at
//  unset focus on this task button

//@-at
//@@c

void taskbutton_unfocus(taskbutton_type* task)
{
  int i;
  GtkStyle* style;

  /* do color stuff */
  style=gtk_style_copy(gtk_widget_get_style(mainwin));
  for(i=0; i<5; i++)
  {
    style->text[i]=config_options.text_color_normal;
    style->fg[i]=config_options.text_color_normal; 
    style->bg[i]=config_options.button_color_normal;
    style->base[i]=config_options.button_color_normal;
  }
  gtk_widget_set_style(task->label, style);
  gtk_widget_set_style(task->button, style);
  gtk_style_unref(style);

  /* show it */
  gtk_widget_show(task->button);
}

//@-body
//@-node:2::taskbutton_unfocus
//@+node:3::taskbutton_iconify
//@+body
//@+at
//  iconify this task button

//@-at
//@@c

void taskbutton_iconify(taskbutton_type* task)
{
  GtkStyle* style;
  int i;
  
  style=gtk_style_copy(gtk_widget_get_style(mainwin));
  for(i=0; i<5; i++)
  {
    style->text[i]=config_options.text_color_iconified;
    style->fg[i]=config_options.text_color_iconified; 
    style->bg[i]=config_options.button_color_iconified;
    style->base[i]=config_options.button_color_iconified;
  }

  gtk_widget_set_style(task->label, style);
  gtk_widget_set_style(task->button, style);
  gtk_style_unref(style);
}

//@-body
//@-node:3::taskbutton_iconify
//@+node:4::taskbutton_normalfy
//@+body
//@+at
//  normal state for task button

//@-at
//@@c

void taskbutton_normalfy(taskbutton_type* task)
{
  GtkStyle* style;
  int i;
  
  style=gtk_style_copy(gtk_widget_get_style(mainwin));
  for(i=0; i<5; i++)
  {
    style->text[i]=config_options.text_color_normal;
    style->fg[i]=config_options.text_color_normal; 
    style->bg[i]=config_options.button_color_normal;
    style->base[i]=config_options.button_color_normal;
  }

  gtk_widget_set_style(task->label, style);
  gtk_widget_set_style(task->button, style);
  gtk_style_unref(style);
}

//@-body
//@-node:4::taskbutton_normalfy
//@-node:3::Task Window State Handlers
//@+node:4::Menu Functions
//@+body
//@+at
// 
// 
// These are the menu callbacks.  They do stuff to the windows of the tasks.
// 
// This is the only place in this section that you will see raw xwindows calls 
// that directly affect other windows.

//@-at
//@-body
//@+node:1::taskbutton_menu_raise
//@+body
//@+at
//  menu handlers

//@-at
//@@c

void taskbutton_menu_raise(GtkWidget* widget, gpointer data)
{
  if(current_task->task->state!=ICONIC)
    XRaiseWindow(current_task->task->display, current_task->task->xwindow);
}

//@-body
//@-node:1::taskbutton_menu_raise
//@+node:2::taskbutton_menu_lower
//@+body
void taskbutton_menu_lower(GtkWidget* widget, gpointer data)
{
  if(current_task->task->state!=ICONIC)
    XLowerWindow(current_task->task->display, current_task->task->xwindow);
}

//@-body
//@-node:2::taskbutton_menu_lower
//@+node:3::taskbutton_menu_focus
//@+body
void taskbutton_menu_focus(GtkWidget* widget, gpointer data)
{
  if(current_task->task->state!=ICONIC)
    XSetInputFocus(current_task->task->display, current_task->task->xwindow, 
		   RevertToPointerRoot, CurrentTime);
}

//@-body
//@-node:3::taskbutton_menu_focus
//@+node:4::taskbutton_menu_iconify
//@+body
void taskbutton_menu_iconify(GtkWidget* widget, gpointer data)
{
  if(current_task->task->state!=ICONIC)
    XIconifyWindow(current_task->task->display, current_task->task->xwindow,
		   DefaultScreen(current_task->task->display));  
  else
    XMapWindow(current_task->task->display, current_task->task->xwindow);
}

//@-body
//@-node:4::taskbutton_menu_iconify
//@+node:5::taskbutton_menu_close
//@+body
void taskbutton_menu_close(GtkWidget* widget, gpointer data)
{
  XEvent xevent = {0,};
  
  xevent.type=ClientMessage;
  xevent.xclient.window= current_task->task->xwindow;
  xevent.xclient.message_type=XInternAtom(current_task->task->display,
					  "WM_PROTOCOLS", FALSE);
  xevent.xclient.format=32;
  xevent.xclient.data.l[0]=XInternAtom(current_task->task->display,
				       "WM_DELETE_WINDOW", FALSE);
  xevent.xclient.data.l[1]=CurrentTime;
  xevent.xclient.data.l[2]=0;
  xevent.xclient.data.l[3]=0;
  xevent.xclient.data.l[4]=0;

  XSendEvent(GDK_DISPLAY(), current_task->task->xwindow, False, 0, &xevent);
}

//@-body
//@-node:5::taskbutton_menu_close
//@+node:6::taskMenu_init
//@+body
//@+at
//  create the menu

//@-at
//@@c

void taskMenu_init()
{
  char buf[50];
  GtkWidget* menu_item;

  taskMenu=gtk_menu_new();

  /* add the raise option */
  sprintf(buf, "Raise");
  menu_item=gtk_menu_item_new_with_label(buf);
  gtk_menu_append(GTK_MENU(taskMenu), menu_item);
  gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			    GTK_SIGNAL_FUNC(taskbutton_menu_raise), NULL);
  gtk_widget_show(menu_item);

  /* add the lower option */  
  sprintf(buf, "Lower");
  menu_item=gtk_menu_item_new_with_label(buf);
  gtk_menu_append(GTK_MENU(taskMenu), menu_item);
  gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			    GTK_SIGNAL_FUNC(taskbutton_menu_lower), NULL);
  gtk_widget_show(menu_item);

  /* add the focus option */  
  sprintf(buf, "Focus");
  menu_item=gtk_menu_item_new_with_label(buf);
  gtk_menu_append(GTK_MENU(taskMenu), menu_item);
  gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			    GTK_SIGNAL_FUNC(taskbutton_menu_focus), NULL);
  gtk_widget_show(menu_item);

  /* add the minimize option */ 
  sprintf(buf, "(Un)Iconify");
  menu_item=gtk_menu_item_new_with_label(buf);
  gtk_menu_append(GTK_MENU(taskMenu), menu_item);
  gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			    GTK_SIGNAL_FUNC(taskbutton_menu_iconify), NULL);
  gtk_widget_show(menu_item);

  /* add the close option */
  sprintf(buf, "Close");
  menu_item=gtk_menu_item_new_with_label(buf);
  gtk_menu_append(GTK_MENU(taskMenu), menu_item);
  gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			    GTK_SIGNAL_FUNC(taskbutton_menu_close), NULL);
  gtk_widget_show(menu_item);

  /* add the main menu */
  sprintf(buf, "Main");
  menu_item=gtk_menu_item_new_with_label(buf);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), GeneralMenu);
  gtk_menu_append(GTK_MENU(taskMenu), menu_item);
  gtk_widget_show(menu_item);
}

//@-body
//@-node:6::taskMenu_init
//@-node:4::Menu Functions
//@+node:5::Button Functions and Calbacks
//@+node:1::taskbutton_size_handler
//@+body
//@+at
//  in non scrolled mode, call this after all displays and updates to correct 
// the size

//@-at
//@@c

void taskbutton_size_handler()
{
  int count; /* number of columns */
  GList* cur;

  /* calculate number of columns */ 
  count=taskbutton_count;
  if(count > 2)
    count=(count/2) + (count%2);
  else
    count/=2;

  if(!count)
    return;


  taskbutton_height=-1;
  taskbutton_width=view->allocation.width/count;

  /* apply changes */
  cur=tasklist;
  while(cur)
  {
    if(((task_type*)cur->data)->button &&((task_type*)cur->data)->button->button)
    {
      gtk_widget_set_usize(((task_type*)cur->data)->button->button, 
                           taskbutton_width, 
                           taskbutton_height);
    }
    cur=cur->next;
  }
  gtk_widget_show_all(mainwin);
}

//@-body
//@-node:1::taskbutton_size_handler
//@+node:2::taskbutton_clicked
//@+body
//@+at
//  handle clicks

//@-at
//@@c

gboolean taskbutton_clicked(GtkWidget* button,GdkEventButton *event, gpointer data)
{
  taskbutton_type* task;

  task=data;

  /* raise and focus on a single leftclick  or pop up a menu on the right click*/
  if(event->type==GDK_BUTTON_PRESS)
  {
    if(event->button==3)
    {
      current_task=task;
      gtk_menu_popup(GTK_MENU(taskMenu), NULL, NULL, NULL, NULL,
		     event->button, event->time);
    }
    else if(task->task->state==ICONIC)
    {
      XMapWindow(task->task->display, task->task->xwindow);
    }
    else
    {
      XRaiseWindow(task->task->display, task->task->xwindow);
      XSetInputFocus(task->task->display, task->task->xwindow, 
	  	     RevertToPointerRoot, CurrentTime);
    }
    return TRUE;
  }
  
  /* iconify on double click */
  if(event->type==GDK_2BUTTON_PRESS)
  {
    XIconifyWindow(task->task->display, task->task->xwindow,
		   DefaultScreen(task->task->display));  
    return TRUE;
  }

  return FALSE;
}

//@-body
//@-node:2::taskbutton_clicked
//@+node:3::taskbutton_new
//@+body
//@+at
//  creates a taskbutton

//@-at
//@@c

taskbutton_type* taskbutton_new(task_type* task)
{
  taskbutton_type* result;
  GdkPixmap* pixmap;
  GdkBitmap* mask;
  GtkWidget* hbox;
  GtkWidget* icon;
  GdkPixbuf* pixbuf;
  GdkPixbuf* pixbuf2;
  int w, h;


  result=malloc(sizeof(taskbutton_type));
  result->pressed_handler=0;
  result->release_handler=0;
  result->leave_handler=0;
  result->task=task;

  /* make the label */
  result->label=gtk_label_new(task->name);

  /* make the button */
  result->button=gtk_button_new();

  /* make the hbox */
  hbox=gtk_hbox_new(0,4);
 
  /* get the pixmap */ 
  task_get_mini_icon(task, &pixmap, &mask, &w, &h);

  /* if the pixmap is there, make a widget and put it in the button's box */
  if(pixmap && mask)
  {
    /* scale the icon to 16x16 */
    pixbuf= gdk_pixbuf_get_from_drawable(NULL, 
					 pixmap,
					 gtk_widget_get_colormap(mainwin),
					 0, 0,
					 0, 0,
					 w, h);

    gdk_pixmap_unref(pixmap);
    gdk_bitmap_unref(mask);

    pixbuf2=gdk_pixbuf_scale_simple(pixbuf, 16, 16, GDK_INTERP_BILINEAR);

    gdk_pixbuf_render_pixmap_and_mask(pixbuf2, &pixmap, &mask, 127);   

    gdk_pixbuf_unref(pixbuf);
    gdk_pixbuf_unref(pixbuf2);

    icon=gtk_pixmap_new(pixmap, mask);
    gtk_box_pack_start(GTK_BOX(hbox), icon, 0,0,0); 
  }
  else
  {
    /* create unknown icon if necessary */
    if(!unknown_pixmap && !unknown_bitmap)
    {
      unknown_pixmap=gdk_pixmap_colormap_create_from_xpm_d(NULL, 
						 gtk_widget_get_colormap(mainwin),
							   &unknown_bitmap, NULL,
							   (gchar**)unknown_xpm);
    }
    icon=gtk_pixmap_new(unknown_pixmap, unknown_bitmap);
    gtk_box_pack_start(GTK_BOX(hbox), icon, 0,0,0); 
  }

  /* pack the label */
  gtk_box_pack_start(GTK_BOX(hbox), result->label, 0,0,0);

  /* add the box */
  gtk_container_add(GTK_CONTAINER(result->button), hbox);


  /*set the click handler */
  gtk_signal_connect(GTK_OBJECT(result->button), "button-press-event",
                     GTK_SIGNAL_FUNC(taskbutton_clicked), result);


  gtk_widget_show_all(result->button);
  /* return button */
  return result;
}

//@-body
//@-node:3::taskbutton_new
//@+node:4::taskbutton_display
//@+body
//@+at
//  adds the taskbutton to a vbox and displays it

//@-at
//@@c

void taskbutton_display(taskbutton_type* task)
{
  GList* child;
  GList* vblist;
  GtkWidget* vbox;  /* a vbox */

  taskbutton_count++;

  child=gtk_container_children(GTK_CONTAINER(mainbox));

  /* handle the easy case */
  if(!child)
  {
    task->box=gtk_vbox_new(0, 0);
    gtk_box_pack_start(GTK_BOX(task->box), task->button, TRUE, TRUE, 0); 
    gtk_box_pack_start(GTK_BOX(mainbox), task->box, 0, 0, 0);
    gtk_widget_show(task->button);
    gtk_widget_show(task->box);
    gtk_widget_show(mainbox);
    gtk_widget_show_all(mainwin);
    return;
  }
  
  while(child)
  {
    vbox=child->data;
    vblist=gtk_container_children(GTK_CONTAINER(vbox));

    /* if this has an available slot, use it.  otherwise, move on */
    if(g_list_position(vblist, g_list_last(vblist)) < 1)
      break;
    child=child->next;
    vbox=0;
  }

  /* if no vbox was found, make a new one */
  if(!vbox)
  {
    vbox=gtk_vbox_new(0, 0);
    gtk_box_pack_start(GTK_BOX(mainbox), vbox, 0, 0, 0);
  }

  task->box=vbox;

  gtk_box_pack_start(GTK_BOX(task->box), task->button, TRUE, TRUE, 0); 
  gtk_widget_show(task->button);
  gtk_widget_show(task->box);
  gtk_widget_show(mainbox);

  if(config_options.mode!=SCROLLED)
    taskbutton_size_handler();
}

//@-body
//@-node:4::taskbutton_display
//@+node:5::taskbutton_destroy
//@+body
void taskbutton_destroy(taskbutton_type* task)
{
  GList* child;
  GList* vbchild;

  taskbutton_count--;
  if(config_options.mode!=SCROLLED)
    taskbutton_size_handler();

  child=gtk_container_children(GTK_CONTAINER(mainbox)); 
  child=g_list_find(child, task->button->parent); 
  gtk_widget_destroy(task->button);
  task->button=0;


  while(child->next)
  {
    /* move it back */
    vbchild=gtk_container_children(GTK_CONTAINER(child->next->data));

    gtk_widget_ref(GTK_WIDGET(vbchild->data));    
    gtk_container_remove(GTK_CONTAINER(child->next->data), 
      GTK_WIDGET(vbchild->data));
    gtk_box_pack_start(GTK_BOX(child->data), GTK_WIDGET(vbchild->data), 
                       TRUE, TRUE, 0);
    gtk_widget_unref(GTK_WIDGET(vbchild->data));
    child=child->next;
  }
  child=gtk_container_children(GTK_CONTAINER(mainbox));
  child=g_list_last(child);
   if(!gtk_container_children(GTK_CONTAINER(child->data)))
     gtk_widget_destroy(GTK_WIDGET(child->data));  
  free(task);

}

//@-body
//@-node:5::taskbutton_destroy
//@+node:6::update_task_display
//@+body
//@+at
//  update's task info on display

//@-at
//@@c

void update_task_display(taskbutton_type* task)
{
  gtk_label_set_text(GTK_LABEL(task->label), (gchar*) task->task->name);
}

//@-body
//@-node:6::update_task_display
//@-node:5::Button Functions and Calbacks
//@-others


//@-body
//@-node:0::@file src/task-gui.c
//@-leo
