/*
    rox-tasklist    provides a tasklist for the rox desktop
    Copyright (C) 2002  Robert Lowe

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    The author may be contacted at pngwen@acm.org
*/
#include "globals.h"

#ifndef XFUNCTIONS_DEFINED
#define XFUNCTIONS_DEFINED

#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include <X11/Xmd.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <stdio.h>
#include <stdlib.h>



typedef enum {NORMAL, ICONIC, FOCUSED, SHADED} win_state;  /*window state type*/
struct taskbutton_type;

struct task_type
{
  unsigned char* name;  /*name of the task*/
  unsigned char* icon;  /*icon name*/
  win_state state; /*state of the task */
  Window xwindow;  /* the window of the task */
  Display* display; /* the display of the task */
  GdkWindow* gdkframe;  /* frame for gdk filtering */
  int visited;          /* tells if it has been seen in a client list */
  struct taskbutton_type* button;
};

typedef struct task_type task_type;

/* prototypes */

void add_task(task_type* task);

GdkWindow*
gdk_window_ref_from_xid (Window xwin);

/*checks to see if the client is an actual xwindow client.  Also grabs the 
  state of the window.  for the task */
gboolean
check_client (Atom state_atom, task_type* task);

/* finds the client and sets the window in the task */
void find_input_client(Atom state_atom, task_type* task);

GdkFilterReturn 
task_event_monitor(GdkXEvent *gdk_xevent,
                   GdkEvent *event,
                   gpointer taskptr);

GdkFilterReturn 
root_event_monitor (GdkXEvent *gdk_xevent,
		    GdkEvent  *event,
		    gpointer   taskptr);

/* update/get the task's info */
void update_task(task_type* task);


/* gets all the information for a task.  The xwindow field of task must be set.
   This will find the client window and retrieve all the needed info  
   it also sets up the filters for the task and adds it to the task list       */
void add_task(task_type* task);

/* remove's a task from the tasklist */
void remove_task(task_type* task);


/* builds the initial tasklist */
void make_task_list(int mode);


/* a gcompare that compares elem to user.  like strcmp */
gint task_compare(gconstpointer elem, gconstpointer user);



//inits x stuff and returns the default tasklist
GList* xfun_init();


void
task_get_mini_icon (task_type *task,
		    GdkPixmap **pixmap,
		    GdkBitmap **mask, int* w, int* h);

gpointer
get_typed_property_data (Display *xdisplay,
			 Window   xwindow,
			 Atom     property,
			 Atom     requested_type,
			 gint    *size_p,
			 guint    expected_format);
/* resets all the visited info */
void reset_visited();
#endif
